/**************************************************************\
 * This file contains all the functions for Lab4 except 
 * sineGen(). Most of it has already been written for you due
 * to the limited time available in a one-day workshop.
 *
 * We have left a few "???" places, though, where you must enter
 * the appropriate C code. Have fun!
\**************************************************************/

/**************************************************************\
 *  Prototypes
\**************************************************************/
void codec_out(short x);
short codec_in(void);
void codec_init(void);
void init_HWI(void);
void emif_init(void);
float lms(float input);
float sineGen(void)

/**************************************************************\
 *  Include files
\**************************************************************/
#include <c6x.h>           // C6000 compiler definitions

#include <csl.h>           // CSL headers
#include <csl_irq.h>
#include <csl_mcbsp.h>

#include <bsl.h>           // BSL headers
#include <bsl_ad535.h>     // -i build option allows CCS to find these headers
#include <bsl_suggestions.h>

/**************************************************************\
 *  Global Variables
\**************************************************************/
    /**********************************************************\
     *  The following AD535 Handle and Config structure were   
     *  entered verbatim from the BSL documentation. The config
     
     0
     *  values are common to many applications; if nothing 
     *  else, they make a good starting point.               
    \**********************************************************/
    AD535_Handle hAD535;                                       
    AD535_Config my_AD535_Config = {                           
        AD535_LOOPBACK_DISABLE,                                
        AD535_MICGAIN_OFF,                                     
        AD535_GAIN_0DB,                                        
        AD535_GAIN_0DB                                         
    };                                                         

/**************************************************************\
 *  Main Routine
\**************************************************************/
void main()
{
    CSL_init();              // Initialize CSL library
    BSL_init();              // Initialize BSL library
    emif_init();             // Dummy function - see note below
    codec_init();
    init_HWI();
 
    AD535_read(hAD535);  /* Write anything to cause mcbsp
                                to start transmit interrupts */
                                
  

    /**************************************************\
     *  Rather than creating an infinite loop in main,
     *  DSP/BIOS provides an infinite idle (IDL) loop.
     *  All you need to do is return from main. We'll
	 *  use IDL in Lab 5 and 6 by returning from main()
     *  to DSP/BIOS's IDL function.
    \**************************************************/                  
    while(1);
}

/**************************************************************\
 *	Configure External Memory Interface (EMIF)
 *
 *  We have a dummy function here to setup the EMIF. In a "real"
 *  system you would be required to setup the EMIF but in our
 *  development system CCS sets up the memory for us (refer to 
 *  the DSK6211_6711.gel file for the CCS script that 
 *  accomplishes this task.
\**************************************************************/
void emif_init(void){
}

/*************************************************************\
 *	Initialize hardware interrupts. 
 **************************************************************
 *  The AD535 codec is hardwired to McBSP0. Therefore,
 *	our system uses the McBSP0 transmit interrupt 
 *  (XINT0) to signal when to write the next output sample.
 *	
 *	Enabling the CPU to recognize the XINT0 interrupt requires:
 *
 *	1. Map McBSP0 transmit interrupt (XINT0) to interrupt 9 (HWI9)
 *     using the Config Tool
 *	2. Enable interrupts globally
 *	3. Enable McBSP transmit interrupt
\*************************************************************/
void init_HWI(void)
{
	IRQ_globalEnable();             // Enable ints globally
	IRQ_enable(IRQ_EVT_RINT0);		// Enable McBSP0 interrupt
}

float buffer[256];
int p=0;

//Interrupt service routine you need to reference in HWI module of CDB file
void RINT0_HWI(void)
{
	float x,y;
	short output;
	short input;
	
	input=codec_in();
	x=(float) input;
	y= lms(x);
	
	buffer[p]=y;
	p=(p+1)%256;
	
	y=y*32768/5;
	output=(short) y;
	codec_out(output); 
	
	
	                    // Outputs sine sample for each XINT0 interrupt?
}

/**********************************************************\
 *	Codec Functions
\**********************************************************/
void codec_init()
{
	/* Use the BSL routines to:  Open, reset, & config the AD535               *\
	 * Part of the AD535_open() routine opens and configures the McBSP0, since *
	\*  all communications with the AD535 codec are handled via the McBSP0.    */
	hAD535 = AD535_open(AD535_localId);      // ???  Open AD535 and set hAD535 handle
	AD535_reset(hAD535);                     // ???  Reset AD535                     
	AD535_config(hAD535, &my_AD535_Config);  // ???  Configure AD535                 
	
	/**************************************************\
	 *	This sets a bit in a McBSP register thus
	 *	allowing it to continue running even when the
	 *	debugger is halted. This keeps the codec from
	 *	being corrupted.
	 *
	 *  We've put this function in the "bsl_suggestions"
	\**************************************************/
	MCBSP_setfree(0);
}

/******************************************************\
 *	Output a sine sample from the function SineGen()
 *	to the AD535 codec.
\******************************************************/
void codec_out(short x) 
{ 
	AD535_HWI_write(hAD535, x);
}

short codec_in() 
{ 
	return AD535_HWI_read(hAD535);
}







